Public Class TreeNodeListBox

    Private WithEvents m_Items As TreeNodeListBoxList
    Private WithEvents m_TreeView As System.Windows.Forms.TreeView
    Private WithEvents m_VBar As VScrollBar

    Private m_InternScrolling As Boolean = False
    Private m_InternSelection As Boolean = False

    Public Sub SetItems(ByVal NodesToDisplay As List(Of TreeNode))
        m_Items.Clear()
        For Each Nd As TreeNode In NodesToDisplay
            m_Items.Add(New TreeNodeListBoxItem(Nd))
        Next
        Dim g As Graphics = Me.CreateGraphics
        m_Items.Recalculate(g)

        SetScrollBar()

        If m_VBar.Enabled Then
            If m_TreeView IsNot Nothing AndAlso m_TreeView.SelectedNode IsNot Nothing AndAlso NodesToDisplay.Contains(m_TreeView.SelectedNode) Then
                For Each Itm As TreeNodeListBoxItem In m_Items
                    If Itm.Node.IsSelected Then
                        m_InternScrolling = True
                        m_VBar.Value = Math.Min(m_Items.Height - Me.Height, Itm.Top)
                        m_InternScrolling = False
                        Exit For
                    End If
                Next
            End If
        End If

        Invalidate()
    End Sub
  

    Public Sub New()

        ' Dieser Aufruf ist für den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Fügen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.
        m_VBar = New VScrollBar
        m_VBar.Dock = DockStyle.Right
        m_VBar.Enabled = False
        Me.Controls.Add(m_VBar)
        m_Items = New TreeNodeListBoxList(Me, Me.Width - m_VBar.Width)
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint _
        Or ControlStyles.AllPaintingInWmPaint, True)
    End Sub


#Region " Properties "
    Property TreeView() As System.Windows.Forms.TreeView
        Get
            Return m_TreeView
        End Get
        Set(ByVal Value As System.Windows.Forms.TreeView)
            m_TreeView = Value
        End Set
    End Property
#End Region


    Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseDown(e)

        'make sure we receive key events
        Me.Focus()

        If e.Button <> MouseButtons.Left Then
            Return
        End If
        'determine which item was clicked
        Dim index As Integer = ItemHitTest(e.X, e.Y)
        If index < 0 Then
            Return
        End If

        Dim TNI As TreeNodeListBoxItem = m_Items(index)

        If Not IsNothing(TNI.Node.TreeView) Then
            m_InternSelection = True
            TNI.Node.TreeView.SelectedNode = TNI.Node
            TNI.Node.Expand()
            TNI.Node.EnsureVisible()
            m_InternSelection = False
        End If


        Me.Invalidate()
    End Sub
    Private Function ItemHitTest(ByVal X As Integer, ByVal Y As Integer) As Integer
        Dim posY As Integer = m_VBar.Value * -1
        Dim i As Integer
        If m_Items.Count > 0 Then
            For i = 0 To m_Items.Count - 1
                posY += m_Items(i).Height

                If Y < posY Then
                    Return i
                End If
            Next i
        End If

        Return -1
    End Function 'ItemHitTest


    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)

        'Benutzerdefinierten Zeichnungscode hier einfügen
        Dim g As Graphics = e.Graphics
        g.Clear(SystemColors.Window)

        Dim MyBottom As Integer = m_VBar.Value + Me.Height
        Dim MyWidth As Integer = Me.Width - m_VBar.Width

        Dim IL As ImageList = Nothing
        Dim Ft As Font = Me.Font
        If m_TreeView IsNot Nothing Then
            IL = m_TreeView.ImageList
        End If

        For Each Itm As TreeNodeListBoxItem In m_Items
            If Itm.Top < MyBottom AndAlso Itm.Top + Itm.Height > m_VBar.Value Then
                Dim Bounds As New Rectangle(0, Itm.Top - m_VBar.Value, MyWidth, Me.Height)
                Itm.Draw(g, IL, Bounds, Ft)
            End If
        Next

    End Sub

    Private Sub TreeNodeListBox_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        m_Items.Width = Me.Width - m_VBar.Width
        SetScrollBar()
        Invalidate()
    End Sub

    Private Sub m_VBar_Scroll(ByVal sender As Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles m_VBar.Scroll
        If m_InternScrolling Then Exit Sub
        Invalidate()
    End Sub


    Private Sub SetScrollBar()
        Dim WasIntern As Boolean = m_InternScrolling
        m_InternScrolling = True

        If m_Items.Height > Me.Height Then
            Dim max As Integer = m_Items.Height - Me.Height
            m_VBar.Enabled = True
            m_VBar.LargeChange = Me.Height
            m_VBar.Maximum = m_Items.Height
            Dim MaxDisplayValue = m_Items.Height - Me.Height
            m_VBar.Value = Math.Min(m_VBar.Value, MaxDisplayValue)
        Else
            m_VBar.Enabled = False
            m_VBar.Value = 0
        End If

        If Not WasIntern Then
            m_InternScrolling = False
        End If
    End Sub
End Class
